package com.yanzuoguang.util.base;


/**
 * 通信格式转换
 * <p>
 * Java和一些windows编程语言如c、c++、delphi所写的网络程序进行通讯时，需要进行相应的转换 高、低字节之间的转换
 * windows的字节序为低字节开头 linux,unix的字节序为高字节开头 java则无论平台变化，都是高字节开头
 */
public class ByteConvertHelper {

    // --------------------------------------- toLH -------------------------------------

    /**
     * 将 long 值转成低字节在前，高字节在后的byte数组
     *
     * @param n   需要转换的值
     * @param max 字节最大长度
     * @return
     */
    public static byte[] toLH(long n, int max) {
        byte[] b = new byte[max];
        for (int i = 0; i < max; i++) {
            int offset = i * 8;
            b[i] = (byte) (n >> offset & 0xff);
        }
        return b;
    }

    /**
     * 将 long 值转成低字节在前，高字节在后的byte数组
     *
     * @param n 需要转换的值
     * @return
     */
    public static byte[] toLH(long n) {
        return toLH(n, 8);
    }

    /**
     * 将int转为低字节在前，高字节在后的byte数组
     *
     * @param n int
     * @return byte[]
     */
    public static byte[] toLH(int n) {
        return toLH(n, 4);
    }

    /**
     * 将short转为低字节在前，高字节在后的byte数组
     *
     * @param n short
     * @return byte[]
     */
    public static byte[] toLH(short n) {
        return toLH(n, 2);
    }

    /**
     * 将float转为低字节在前，高字节在后的byte数组
     */
    public static byte[] toLH(float f) {
        return toLH(Float.floatToRawIntBits(f));
    }

    // --------------------------------------- toHL -------------------------------------

    /**
     * 将 long 值转成高字节在前，低字节在后的byte数组
     *
     * @param n   需要转换的值
     * @param max 字节最大长度
     * @return
     */
    public static byte[] toHL(long n, int max) {
        byte[] b = new byte[max];
        for (int i = 0; i < max; i++) {
            int offset = (max - 1 - i) * 8;
            b[i] = (byte) (n >> offset & 0xff);
        }
        return b;
    }

    /**
     * 将 long 值转成高字节在前，低字节在后的byte数组
     *
     * @param n 需要转换的值
     * @return
     */
    public static byte[] toHL(long n) {
        return toHL(n, 8);
    }

    /**
     * 将int转为高字节在前，低字节在后的byte数组
     *
     * @param n int
     * @return byte[]
     */
    public static byte[] toHL(int n) {
        return toHL(n, 4);
    }

    /**
     * 将short转为高字节在前，低字节在后的byte数组
     *
     * @param n short
     * @return byte[]
     */
    public static byte[] toHL(short n) {
        return toHL(n, 2);
    }

    /**
     * 将float转为高字节在前，低字节在后的byte数组
     */
    public static byte[] toHL(float f) {
        return toHL(Float.floatToRawIntBits(f));
    }


    // --------------------------------------- byLH -------------------------------------

    /**
     * 将低字节转换为长整形
     *
     * @param bytes 字节
     * @param off   开始位置
     * @param max   最大位置
     * @return
     */
    public static long toLongByLH(byte[] bytes, int off, int max) {
        long ret = 0;
        for (int i = 0; i < max; i++) {
            int b = bytes[off + i] & 0xFF;
            int offset = i * 8;
            ret |= (b << offset);
        }
        return ret;
    }

    /**
     * 将低字节数组转换为long
     *
     * @param b byte[]
     * @return long
     */
    public static long toLongByLH(byte[] b) {
        return toLongByLH(b, 0);
    }

    /**
     * 将低字节数组转换为long
     *
     * @param b
     * @param off
     * @return
     */
    public static long toLongByLH(byte[] b, int off) {
        return toLongByLH(b, off, 8);
    }

    /**
     * 将低字节数组转换为int
     *
     * @param b byte[]
     * @return int
     */
    public static int toIntByLH(byte[] b) {
        return toIntByLH(b, 0);
    }

    /**
     * 将低字节数组转换为int
     *
     * @param b
     * @param off
     * @return
     */
    public static int toIntByLH(byte[] b, int off) {
        return (int) toLongByLH(b, off, 4);
    }

    /**
     * 低字节数组到short的转换
     *
     * @param b
     * @return
     */
    public static short toShortByLH(byte[] b) {
        return toShortByLH(b, 0);
    }

    /**
     * 低字节数组到short的转换
     *
     * @param b
     * @param off
     * @return
     */
    public static short toShortByLH(byte[] b, int off) {
        return (short) toLongByLH(b, off, 2);
    }

    /**
     * 低字节数组转换为float
     *
     * @param b byte[]
     * @return float
     */
    public static float toFloatByLH(byte[] b) {
        int i = toIntByLH(b);
        return Float.intBitsToFloat(i);
    }


    // --------------------------------------- byHL -------------------------------------

    /**
     * 将高字节转换为长整型
     *
     * @param bytes 字节
     * @param off   开始位置
     * @param max   最大位置
     * @return
     */
    public static long toLongByHL(byte[] bytes, int off, int max) {
        long ret = 0;
        for (int i = 0; i < max; i++) {
            int b = bytes[off + i] & 0xFF;
            int offset = (max - 1 - i) * 8;
            ret |= (b << offset);
        }
        return ret;
    }

    /**
     * 将高字节数组转换为long
     *
     * @param b
     * @return
     */
    public static long toLongByHL(byte[] b) {
        return toLongByHL(b, 0);
    }

    /**
     * 将高字节数组转换为long
     *
     * @param b   byte[]
     * @param off long
     * @return long
     */
    public static long toLongByHL(byte[] b, int off) {
        return toLongByHL(b, off, 8);
    }


    /**
     * 将高字节数组转换为int
     *
     * @param b
     * @return
     */
    public static int toIntByHL(byte[] b) {
        return toIntByHL(b, 0);
    }

    /**
     * 将高字节数组转换为int
     *
     * @param b   byte[]
     * @param off int
     * @return int
     */
    public static int toIntByHL(byte[] b, int off) {
        return (int) toLongByHL(b, off, 4);
    }


    /**
     * 高字节数组到short的转换
     *
     * @param b
     * @return
     */
    public static short toShortByHL(byte[] b) {
        return toShortByHL(b, 0);
    }

    /**
     * 高字节数组到short的转换
     *
     * @param b
     * @param off
     * @return
     */
    public static short toShortByHL(byte[] b, int off) {
        return (short) toLongByHL(b, off, 2);
    }


    /**
     * 高字节数组转换为float
     *
     * @param b byte[]
     * @return float
     */
    public static float toFloatByHL(byte[] b) {
        int i = toIntByHL(b);
        return Float.intBitsToFloat(i);
    }

    /**
     * 获取短整形的字节,高字节在前，低字节在后的byte数组
     *
     * @param value
     * @return
     */
    public static byte[] getBytes(short value) {
        return toHL(value);
    }

    /**
     * 获取短整形的字节,高字节在前，低字节在后的byte数组
     *
     * @param value
     * @return
     */
    public static byte[] getBytes(int value) {
        return toHL(value);
    }

    /**
     * 获取短整形的字节,高字节在前，低字节在后的byte数组
     *
     * @param value
     * @return
     */
    public static byte[] getBytes(long value) {
        return toHL(value);
    }

    /**
     * 将byte数组中的元素倒序排列
     */
    public static byte[] bytesReverseOrder(byte[] b) {
        int length = b.length;
        byte[] result = new byte[length];
        for (int i = 0; i < length; i++) {
            result[length - i - 1] = b[i];
        }
        return result;
    }

    /**
     * 将本机的 short 转换成网络上传输支持的 short ,用于多语言通讯时处理, 将 short 类型的值转换为字节序颠倒过来对应的 short 值
     *
     * @param from
     * @return
     */
    public static short reverse(short from) {
        return toShortByHL(toLH(from));
    }

    /**
     * 将本机的 int 转换成网络上传输支持的 int ,用于多语言通讯时处理, 将 int 类型的值转换为字节序颠倒过来对应的 int 值
     *
     * @param from
     * @return
     */
    public static int reverse(int from) {
        return toIntByHL(toLH(from));
    }

    /**
     * 将本机的 long 转换成网络上传输支持的 long ,用于多语言通讯时处理, 将 long 类型的值转换为字节序颠倒过来对应的 long 值
     *
     * @param from
     * @return
     */
    public static long reverse(long from) {
        return toLongByHL(toLH(from));
    }

    /**
     * 将本机的 float 转换成网络上传输支持的 float ,用于多语言通讯时处理, 将 float 类型的值转换为字节序颠倒过来对应的 float 值
     *
     * @param f float
     * @return float
     */
    public static float reverse(float f) {
        float result = toFloatByHL(toLH(f));
        return result;
    }

    /**
     * 将String转为byte数组
     */
    public static byte[] stringToBytes(String s, int length) {
        while (s.getBytes().length < length) {
            s += " ";
        }
        return s.getBytes();
    }

    /**
     * 将字节数组转换为String
     *
     * @param b byte[]  需要转换的字节,如 [0x00,0x01]
     * @return String 返回转换成功的字符串 如: 0001
     */
    public static String bytesToString(byte[] b) {
        StringBuffer result = new StringBuffer();
        int length = b.length;
        for (int i = 0; i < length; i++) {
            result.append((char) (b[i] & 0xff));
        }
        return result.toString();
    }

    /**
     * 将字符串转换为byte数组
     *
     * @param s String
     * @return byte[]
     */
    public static byte[] stringToBytes(String s) {
        return s.getBytes();
    }

    /**
     * 将字节转换为字节字符串。如 [0x00,0x01]转成成 00 01
     *
     * @param bb
     * @return
     */
    public static String logBytes(byte[] bb) {
        int length = bb.length;
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length; i++) {
            sb.append(bb + " ");
        }
        return sb.toString();
    }

//    public static void main(String[] args) {
//        long old = 99L;
//        long from = reverse(old);
//        long to = reverse(from);
//        System.out.println(String.format("old:%d %d %d", old, from, to));
//    }
}
